/* Copyright (c) 2014, Giovanni Spina <giovanni@codeforfun.it>
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the uploader nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL GIOVANNI SPINA BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.*/

#ifndef __LCD_DRIVER_H__
#define __LCD_DRIVER_H__

#include <Types.h>

#define COLOR8			0x02
#define COLOR12			0x03
#define COLOR16			0x05

/* RGB8 colors */
#define RGB_WHITE8		0xFF
#define RGB_BLACK8		0x00
#define RGB_RED8		0xE0
#define RGB_BLUE8		0x03
#define RGB_LIGHTBLUE8	0x0F
#define RGB_YELLOW8		0xFC
#define RGB_ORANGE8		0xF0
#define RGB_GREEN8		0x58
#define RGB_PURPLE8		0x22
#define RGB_LIGHTGREEN8	0x78
#define RGB_AZURE8		0x37
#define RGB_GREY8		0xBB

/* RGB12 colors */
#define RGB_WHITE		0x0FFF
#define RGB_BLACK		0x0000
#define RGB_RED			0x0F00
#define RGB_GREEN		0x00F0
#define RGB_BLUE		0x000F
#define RGB_PURPLE		0x0E0E
#define RGB_CYAN		0x00FF
#define RGB_MAGENTA		0x0F0F
#define RGB_YELLOW		0x0FF0
#define RGB_BROWN		0x0B22
#define RGB_ORANGE		0x0FA0
#define RGB_PINK		0x0F6A

/* RGB16 colors */
#define RGB_WHITE16		0xFFFF
#define RGB_BLACK16		0x0000
#define RGB_RED16		0xF800
#define RGB_GREEN16		0x07E0
#define RGB_BLUE16		0x001F
#define RGB_PURPLE16	0xA11E
#define RGB_CYAN16		0x07FF
#define RGB_MAGENTA16	0xF81F
#define RGB_YELLOW16	0xFFE0 
#define RGB_BROWN16		0xA145
#define RGB_ORANGE16	0xFBE0
#define RGB_PINK16		0xFDB8

#define RDDIDIF			0x04
#define RDID1			0xDA	// read ID1 
#define RDID2			0xDB	// read ID2 
#define RDID3			0xDC	// read ID3 

#define ENDPAGE			132
#define ENDCOL			130

/* EPSON Controller Definitions */
#define DISON			0xAF	// Display on
#define DISOFF			0xAE	// Display off
#define DISNOR			0xA6	// Normal display
#define DISINV			0xA7	// Inverse display
#define SLPIN			0x95	// Sleep in
#define SLPOUT			0x94	// Sleep out
#define COMSCN			0xBB	// Common scan direction
#define DISCTL			0xCA	// Display control
#define PASET			0x75	// Page address set
#define CASET			0x15	// Column address set
#define DATCTL			0xBC	// Data scan direction, etc.
#define RGBSET8			0xCE	// 256-color position set
#define RAMWR			0x5C	// Writing to memory
#define RAMRD			0x5D	// Reading from memory
#define PTLIN			0xA8	// Partial display in
#define PTLOUT			0xA9	// Partial display out
#define RMWIN			0xE0	// Read and modify write
#define RMWOUT			0xEE	// End
#define ASCSET			0xAA	// Area scroll set
#define SCSTART			0xAB	// Scroll start set
#define OSCON			0xD1	// Internal oscillation on
#define OSCOFF			0xD2	// Internal osciallation off
#define PWRCTR			0x20	// Power control
#define VOLCTR			0x81	// Electronic volume control
#define VOLUP			0xD6	// Increment electronic control by 1
#define VOLDOWN			0xD7	// Decrement electronic control by 1
#define TMPGRD			0x82	// Temperature gradient set
#define EPCTIN			0xCD	// Control EEPROM
#define EPCOUT			0xCC	// Cancel EEPROM control
#define EPMWR			0xFC	// Write into EEPROM
#define EPMRD			0xFD	// Read from EEPROM
#define EPSRRD1			0x7C	// Read register 1
#define EPSRRD2			0x7D	// Read register 2
#define NOP				0x25	// No op

/* PHILLIPS Controller Definitions */
#define NOPP			0x00	// No operation
#define BSTRON			0x03	// Booster voltage on
#define SLEEPIN			0x10	// Sleep in
#define SLEEPOUT		0x11	// Sleep out
#define NORON			0x13	// Normal display mode on
#define INVOFF			0x20	// Display inversion off
#define INVON			0x21	// Display inversion on
#define SETCON			0x25	// Set contrast
#define DISPOFF			0x28	// Display off
#define DISPON			0x29	// Display on
#define CASETP			0x2A	// Column address set
#define PASETP			0x2B	// Page address set
#define RAMWRP			0x2C	// Memory write
#define RGBSET			0x2D	// Color set
#define MADCTL			0x36	// Memory data access control
#define COLMOD			0x3A	// Interface pixel format
#define DISCTR			0xB9	// Super frame inversion
#define EC				0xC0	// Internal or external oscillator

extern sfr sbit DATA_PIN;
extern sfr sbit SCK_PIN;
extern sfr sbit CS_PIN;
extern sfr sbit RESET_PIN;

extern sfr sbit DATA_PIN_DIRECTION;
extern sfr sbit SCK_PIN_DIRECTION;
extern sfr sbit CS_PIN_DIRECTION;
extern sfr sbit RESET_PIN_DIRECTION;

enum NOKIA_LCD_TYPE {
	EPSON_CONTROLLER, 
	PHILLIPS_CONTROLLER,
	UNKNOWN_CONTROLLER
};

enum FONT_SIZE
{
	SMALL = 0,
	MEDIUM = 1,
	LARGE = 2
};

enum TEXT_JUSTIFY
{
	JUSTIFY_LEFT,
	JUSTIFY_CENTER,
	JUSTIFY_RIGHT
};

typedef struct
{
	uint32 id;
	uint8 width;
	uint8 height;
	uint8 format;
	uint8 type;
	int8 contrast;
} LCDInfo;

typedef struct text_pos
{
uint8 x;
uint8 y;
} text_pos;

void LCD_Init(uint8, uint8);
void LCD_Cmd(uint8);
void LCD_Data(uint8);
void LCD_contrast(int8);
int8 LCD_getContrast();
void LCD_enable();
void LCD_disable();
void LCD_drawPixel(uint8, uint8, uint16);
text_pos LCD_getPixel(uint8, uint8);
void LCD_drawHorizontalLine(uint8, uint8, uint8, uint16);
void LCD_drawVerticalLine(uint8, uint8, uint8, uint16);
void LCD_drawLine(uint8, uint8, uint8, uint8, uint16);
void LCD_drawCircle(uint8, uint8, uint8, uint16);
void LCD_drawFillCircle(uint8, uint8, uint8, uint16);
void LCD_drawRect(uint8, uint8, uint8, uint8, uint16);
void LCD_drawFillRect(uint8, uint8, uint8, uint8, uint16);
void LCD_drawTriangle(uint8, uint8, uint8, uint8, uint8, uint8, uint16);
void LCD_drawFilledTriangle(uint8, uint8, uint8, uint8, uint8, uint8, uint16);
text_pos LCD_drawChar(uint8, uint8, char, uint16, uint16, uint8);
uint8 LCD_getStringWidth(char*, uint8);
uint8 LCD_getRomStringWidth(const char*, uint8);
void LCD_drawString(uint8, uint8, char *str, uint16, uint16, uint8, uint8);
void LCD_drawRomString(uint8, uint8, const char *str, uint16, uint16, uint8, uint8);
void LCD_drawImage(const uint8* im, uint8 x, uint8 y);
uint8 LCD_getImageWidth(const uint8*);
uint8 LCD_getImageHeight(const uint8*);
void LCD_getScreenSize(uint8*, uint8*);
void LCD_clear(uint16);

#endif /* __LCD_DRIVER_H__ */